from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse
from .forms import ProductUploadForm
from django.contrib.auth.models import User, auth
from cart.models import PCart, Quantity
from accounts.models import KisanData
from .models import ProductData, Rating, UsersRated, Category, SubCategory
from django.db.models import Q
from django.contrib import messages
import os
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from accounts.views import kisan_login_required

# Create your views here.
def s1(request):
    return render(request, 'seller/s1.html')


@kisan_login_required
def sel_deshboard(request):
    return render(request, 'kisan/sel_deshboard.html')


@kisan_login_required
def product_upload(request):
    b = list(KisanData.objects.filter(mobile=request.user.username).values_list('name', flat=True))
    if request.method == 'POST':
        form = ProductUploadForm(request.POST, request.FILES)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.seller_name = b[0]
            instance.p_owner = request.user
            instance.save();
            messages.info(request, 'Product uploaded successfully!')
            return redirect('/kisan/my_prod')
        else:
            return render(request, 'kisan/product_upload.html', {'form': form})
    else:
        form = ProductUploadForm()
    return render(request, 'kisan/product_upload.html', {
        'form': form
    })


@kisan_login_required
def profile(request):
    x = KisanData.objects.get(mobile=request.user.username)
    if x:
        return render(request, 'kisan/profile.html', {'data': x})
    return render(request, 'kisan/profile.html')


from accounts.forms import KisanProfileForm
@kisan_login_required
def profile_edit(request):
    obj = get_object_or_404(KisanData, user=request.user)
    form = KisanProfileForm(request.POST or None, request.FILES or None, instance=obj)
    context = {'form': form}
    print(form.is_valid())
    if form.is_valid():
        print('valid')
        obj = form.save(commit=False)
        obj.save()
        messages.info(request, 'Profile updated successfully!')
        context = {'form': form}
        # return render(request, 'seller/product_upload.html', context)
        return redirect('/kisan/profile/edit')

    else:
        context = {'form': form}
        # return render(request, 'seller/edit_prod.html', context)
        return render(request, 'kisan/profile_edit.html', {'data': form})


def search(request):
    query = request.GET.get('q', None)
    x = ProductData.objects.none()

    if query is not None:
        lookups = Q(p_name__icontains=query) | Q(p_desc__icontains=query)
        x = ProductData.objects.filter(lookups).distinct()
        return render(request, 'customer/product_display.html', {'prods': x})
    return render(request, 'customer/product_display.html', {'prods': x})


def browse(request):
    p = ProductData.objects.all()
    sub_cat = SubCategory.objects.filter(category__cat_type='SP')
    query_Set = Category.objects.filter(cat_type='SP')
    page = request.GET.get('page', 1)
    paginator = Paginator(query_Set, 20)
    try:
        cat = paginator.page(page)
    except PageNotAnInteger:
        cat = paginator.page(1)
    except EmptyPage:
        cat = paginator.page(paginator.num_pages)
    return render(request, 'kisan/products_display.html', {'prods': p, 'categories': cat, 'sub_category': sub_cat})

@kisan_login_required
def my_prod(request):
    # s_email = request.user.username
    # p = ProductData.objects.filter(s_username__exact=s_email)
    p = ProductData.objects.filter(p_owner=request.user)
    total = len(list(p.values()))
    return render(request, 'kisan/my_prod.html', {'prods': p, 'total': total})

@kisan_login_required
def editpage(request, object_id):
    obj = get_object_or_404(ProductData, id=object_id)
    form = ProductUploadForm(request.POST or None, request.FILES or None, instance=obj)
    context = {'form': form}

    if request.method == 'POST' and 'deleter' in request.POST:
        file = obj.p_img.path
        ProductData.objects.filter(id=object_id).delete()
        os.remove(file)
        messages.info(request, 'Product Deleted')
        return redirect('/kisan/my_prod')

    if form.is_valid():
        obj = form.save(commit=False)
        obj.save()
        messages.info(request, 'Product updated successfully!')
        context = {'form': form}
        # return render(request, 'seller/product_upload.html', context)
        return redirect('/kisan/my_prod')

    else:
        context = {'form': form}
        return render(request, 'kisan/edit_prod.html', context)

@kisan_login_required
def delete_record(request, object_id):
    obj = get_object_or_404(ProductData, id=object_id)
    file = obj.p_img.path
    ProductData.objects.filter(id=object_id).delete()
    os.remove(file)
    messages.info(request, 'Product Deleted')
    return redirect('/seller')


def desc(request, object_id):
    product = ProductData.objects.get(id=object_id)
    sellers = SellerData.objects.all()
    list_products = ProductData.objects.filter(p_name__icontains=product.p_name).order_by('p_price')
    p1 = ProductData.objects.filter(id=list_products[0].id).values()
    s = p1[0]['seller_name']
    num_res1 = Rating.objects.filter(seller=s).count()
    if num_res1 == 0:
        co = 0
        us = []
    else:
        co = Rating.objects.filter(seller=s).values()[0]['rate']
        us1 = Rating.objects.get(seller=s)
        us = us1.users.values_list()
    if request.method == 'POST' and 'deleter' in request.POST:
        Q = request.POST.get('quant', list_products[0].min_q)
        num_res = PCart.objects.filter(user=request.user.email).count()
        if num_res == 0:
            p = PCart(user=request.user.email)
            p.save()
        q = Quantity(min_q=Q)
        q.save()
        p1 = PCart.objects.get(user__iexact=request.user.email)
        p1.p_name.add(list_products[0])
        p1.quant.add(q)
        p1.save()
    return render(request, 'seller/product_description.html',
                  {'data': product, 'listdata': list_products, 't': 0, 'co': co, 'us': us, 'seller': sellers})


def desc1(request, object_id):
    product = ProductData.objects.get(id=object_id)
    sellers = SellerData.objects.all()
    list_products = ProductData.objects.filter(p_name__icontains=product.p_name).order_by('p_price')
    p1 = ProductData.objects.filter(id=product.id).values()
    s = p1[0]['seller_name']
    num_res1 = Rating.objects.filter(seller=s).count()
    if num_res1 == 0:
        co = 0
    else:
        co = Rating.objects.filter(seller=s).values()[0]['rate']
    if request.method == 'POST' and 'deleter' in request.POST:
        Q = request.POST.get('quant', product.min_q)
        num_res = PCart.objects.filter(user=request.user.email).count()
        if num_res == 0:
            p = PCart(user=request.user.email)
            p.save()
        q = Quantity(min_q=Q)
        q.save()
        p1 = PCart.objects.get(user__iexact=request.user.email)
        p1.p_name.add(product)
        p1.quant.add(q)
        p1.save()
    return render(request, 'seller/product_description.html',
                  {'data': product, 'listdata': list_products, 't': 1, 'co': co, 'seller':sellers})


def get_subcategory(request):
    id = request.GET.get('id', '')
    result = list(SubCategory.objects.filter(category_id=int(id)).values('id', 'cat_title'))
    return HttpResponse(json.dumps(result), content_type="application/json")

def categories_m(request):
    query_Set = Category.objects.all()
    page = request.GET.get('page', 1)
    paginator = Paginator(query_Set, 20)
    try:
        cat = paginator.page(page)
    except PageNotAnInteger:
        cat = paginator.page(1)
    except EmptyPage:
        cat = paginator.page(paginator.num_pages)
    return render(request, 'index.html', {'categories': cat})


def subcategory_product(request, object_id):
    query_Set = ProductData.objects.filter(subcategory=object_id).order_by('p_price')
    subcat_des = SubCategory.objects.get(id=object_id)
    sub_cat = SubCategory.objects.all()
    cat = Category.objects.all()
    page = request.GET.get('page', 1)
    paginator = Paginator(query_Set, 20)
    try:
        prod = paginator.page(page)
    except PageNotAnInteger:
        prod = paginator.page(1)
    except EmptyPage:
        prod = paginator.page(paginator.num_pages)
    return render(request, 'seller/subcat_products_display.html', {'prods': prod, 'categories': cat, 'sub_category': sub_cat, 'subcat_des': subcat_des})

def seller_details(request):
    # seller = SellerData.objects.get(id=1)
    return HttpResponse('Entered OTP is wrong', content_type='text/plain')
